from .jekyll_writer import JekyllWriter, JekyllTranslator
from docutils.io import StringOutput
from os import path
from sphinx.builders import Builder
from sphinx.builders import Builder
from sphinx.locale import __
from sphinx.util import logging
from sphinx.util.osutil import ensuredir, os_path

if False:
    from typing import Any, Dict, Iterator, Set, Tuple
    from docutils import nodes
    from sphinx.application import Sphinx

logger = logging.getLogger(__name__)

class JekyllBuilder(Builder):
    name = 'jekyll'
    format = 'jekyll'
    epilog = __('The jekyll files are in %(outdir)s.')

    out_suffix = '.md'
    allow_parallel = True
    default_translator_class = JekyllTranslator

    current_docname = None

    markdown_http_base = 'https://localhost'

    def init(self):
        self.secnumbers = {}

    def get_outdated_docs(self):
        for docname in self.env.found_docs:
            if docname not in self.env.all_docs:
                yield docname
                continue
            targetname = path.join(self.outdir, docname + self.out_suffix)
            try:
                targetmtime = path.getmtime(targetname)
            except Exception:
                targetmtime = 0
            try:
                srcmtime = path.getmtime(self.env.doc2path(docname))
                if srcmtime > targetmtime:
                    yield docname
            except EnvironmentError:
                pass

    def get_target_uri(self, docname, typ=None):
        return ''

    def prepare_writing(self, docnames):
        self.writer = JekyllWriter(self)

    def write_doc(self, docname, doctree):
        self.current_docname = docname
        self.secnumbers = self.env.toc_secnumbers.get(docname, {})
        destination = StringOutput(encoding='utf-8')
        self.writer.write(doctree, destination)
        outfilename = path.join(self.outdir, os_path(docname) + self.out_suffix)
        ensuredir(path.dirname(outfilename))
        try:
            with open(outfilename, 'w', encoding='utf-8') as f:  # type: ignore
                f.write(self.writer.output)
        except (IOError, OSError) as err:
            logger.warning(__("error writing file %s: %s"), outfilename, err)

    def finish(self):
        pass



from codecs import open
from os import path
from setuptools import setup, find_packages
from subprocess import check_output

here = path.abspath(path.dirname(__file__))

check_output(
    'pandoc --from=markdown --to=rst --output=' + path.join(here, 'README.rst') + ' ' + path.join(here, 'README.md'),
    shell=True
)

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

install_requires = list()
with open(path.join(here, 'requirements.txt'), 'r', encoding='utf-8') as f:
    for line in f.readlines():
        install_requires.append(line)

setup(
    name='sphinx-jekyll-builder',

    version='0.1.1',

    description='sphinx builder that outputs jekyll compatible markdown files with frontmatter',

    long_description=long_description,

    url='https://github.com/codejamninja/sphinx-jekyll-builder',

    author='Jam Risser',

    author_email='jam@codejam.ninja',

    license='MIT',

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='sphinx docs documentation jekyll',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=install_requires,

    include_package_data=True,

    entry_points = {
        'sphinx.builders': [
            'jekyll = sphinx_jekyll_builder',
        ],
    }
)



from sphinx_markdown_builder.markdown_writer import MarkdownTranslator, MarkdownWriter
from munch import munchify
import yaml

class JekyllTranslator(MarkdownTranslator):
    visited_title = False
    title = None

    def visit_document(self, node):
        self.title = getattr(self.builder, 'current_docname')
        super().visit_document(node)

    def depart_document(self, node):
        variables = munchify({
            'current_docname': getattr(self.builder, 'current_docname'),
            'images': getattr(self.builder, 'images'),
            'title': self.title,
            'versioning_method': getattr(self.builder, 'versioning_method')
        })
        variables_yaml = yaml.safe_dump(variables)
        frontmatter = '---\n' + variables_yaml + '---\n'
        self.add(frontmatter, section='head')
        super().depart_document(node)

    def visit_title(self, node):
        if not self.visited_title:
            self.title = node.astext()
        self.visited_title = True
        super().visit_title(node)

    def depart_title(self, node):
        super().depart_title(node)

class JekyllWriter(MarkdownWriter):
    translator_class = JekyllTranslator
